package com.qmth.digit.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.digit.print.business.bean.query.BlurryParam;
import com.qmth.digit.print.business.bean.query.CourseParam;
import com.qmth.digit.print.business.entity.BasicCourse;
import com.qmth.digit.print.business.entity.ExamTask;
import com.qmth.digit.print.business.mapper.BasicCourseMapper;
import com.qmth.digit.print.business.service.BasicCourseService;
import com.qmth.digit.print.business.service.ExamTaskService;
import com.qmth.digit.print.common.enums.ExceptionResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCourseServiceImpl extends ServiceImpl<BasicCourseMapper, BasicCourse> implements BasicCourseService {

    @Autowired
    private ExamTaskService examTaskService;

    @Override
    public IPage<BasicCourse> list(CourseParam param) {
        Page<BasicCourse> page = new Page<>(param.getPageNumber(), param.getPageSize());
        IPage<BasicCourse> courseIPage = this.baseMapper.listPage(page, param);
        return courseIPage;
    }

    @Override
    public List<BasicCourse> list(BlurryParam param) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, param.getSchoolId())
                .and(i -> i.like(BasicCourse::getCode, param.getParam()).or().like(BasicCourse::getName, param.getParam()));
        return this.list(queryWrapper);
    }

    @Override
    public boolean saveCourse(BasicCourse course) {
        QueryWrapper<BasicCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCourse::getSchoolId, course.getSchoolId()).eq(BasicCourse::getCode, course.getCode());
        BasicCourse basicCourse = this.getOne(queryWrapper);

        // 新增
        if (Objects.isNull(course.getId())) {
            if (basicCourse != null) {
                throw ExceptionResultEnum.ERROR.exception("课程已存在");
            }
             course.setCreateTime(System.currentTimeMillis());
        }
        // 修改
        {
            if (basicCourse != null && basicCourse.getId() != course.getId()) {
                throw ExceptionResultEnum.ERROR.exception("课程已存在");
            }

            // 旧code是否已有命题任务
            BasicCourse oldCourse = this.baseMapper.selectById(course.getId());
            List<ExamTask> taskList = examTaskService.listByCourseCode(oldCourse.getSchoolId(), oldCourse.getCode());
            if(taskList != null && taskList.size() > 0){
                throw ExceptionResultEnum.ERROR.exception("课程绑定命题任务，不能修改");
            }
            course.setUpdateTime(System.currentTimeMillis());
        }
        return this.saveOrUpdate(course);
    }

    @Override
    public boolean remove(Long id) {
        // code是否已有命题任务
        BasicCourse course = this.baseMapper.selectById(id);
        List<ExamTask> taskList = examTaskService.listByCourseCode(course.getSchoolId(), course.getCode());
        if(taskList != null && taskList.size() > 0){
            throw ExceptionResultEnum.ERROR.exception("课程绑定命题任务，不能删除");
        }
        return this.removeById(id);
    }
}
