package com.qmth.digit.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.digit.print.business.bean.dto.OrgDto;
import com.qmth.digit.print.business.entity.BasicTemplateOrg;
import com.qmth.digit.print.business.entity.SysOrg;
import com.qmth.digit.print.business.mapper.SysOrgMapper;
import com.qmth.digit.print.business.service.BasicTemplateOrgService;
import com.qmth.digit.print.business.service.SysOrgService;
import com.qmth.digit.print.common.enums.ExceptionResultEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * <p>
 * 学校组织架构 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysOrgServiceImpl extends ServiceImpl<SysOrgMapper, SysOrg> implements SysOrgService {

    @Autowired
    private BasicTemplateOrgService basicTemplateOrgService;

    @Override
    public List<OrgDto> listOrgTree(Long schoolId) {
        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrg::getSchoolId, schoolId).orderByAsc(SysOrg::getParentId, SysOrg::getCode);
        List<SysOrg> orgList = this.list(queryWrapper);
        Map<Long, OrgDto> map = new LinkedHashMap<>();
        for (SysOrg sysOrg : orgList) {
            OrgDto orgDto = new OrgDto();
            orgDto.setId(sysOrg.getId());
            orgDto.setSchoolId(sysOrg.getSchoolId());
            orgDto.setCode(sysOrg.getCode());
            orgDto.setName(sysOrg.getName());
            orgDto.setParentId(sysOrg.getParentId());
            orgDto.setEnable(sysOrg.getEnable());
            map.put(sysOrg.getId(), orgDto);
        }

        Iterator<Long> iterator = map.keySet().iterator();
        Set<Long> deleteKeys = new HashSet<>();
        while (iterator.hasNext()) {
            Long parentId = iterator.next();
            if (map.get(parentId).getParentId() != null) {
                map.get(map.get(parentId).getParentId()).getChildren().add(map.get(parentId));
                deleteKeys.add(parentId);
            }
        }
        for (Long key : deleteKeys) {
            map.remove(key);
        }

        return new ArrayList<>(map.values());
    }

    @Override
    public boolean saveOrg(SysOrg org) {
        QueryWrapper<SysOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysOrg::getSchoolId, org.getSchoolId()).eq(SysOrg::getCode, org.getCode());
        SysOrg sysOrg = this.getOne(queryWrapper);

        boolean isSuccess;

        // 新增
        if (org.getId() == null) {
            if (sysOrg != null) {
                throw ExceptionResultEnum.ERROR.exception("机构代码已存在");
            }
            org.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(org);
        }
        // 修改
        else {
            if (sysOrg != null && org.getId() != sysOrg.getId()) {
                throw ExceptionResultEnum.ERROR.exception("机构代码已存在");
            }
            org.setUpdateTime(System.currentTimeMillis());
            isSuccess = this.updateById(org);
        }

        return isSuccess;
    }

    @Override
    public boolean enable(SysOrg org) {
        /*QueryWrapper<BasicTemplateOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicTemplateOrg::getOrgId, org.getId());
        List<BasicTemplateOrg> basicTemplateOrgs = basicTemplateOrgService.list(queryWrapper);
        if(basicTemplateOrgs != null && basicTemplateOrgs.size() > 0){
            throw ExceptionResultEnum.ERROR.exception("机构已绑定，不能禁用");
        }*/
        UpdateWrapper<SysOrg> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysOrg::getEnable, org.getEnable()).eq(SysOrg::getId, org.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean remove(Long id) {
        QueryWrapper<BasicTemplateOrg> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicTemplateOrg::getOrgId, id);
        List<BasicTemplateOrg> basicTemplateOrgs = basicTemplateOrgService.list(queryWrapper);
        if (basicTemplateOrgs != null && basicTemplateOrgs.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("机构已绑定，不能删除");
        }
        return this.removeById(id);
    }
}
