package com.qmth.digit.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.digit.print.business.bean.query.RoleParam;
import com.qmth.digit.print.business.entity.SysRole;
import com.qmth.digit.print.business.entity.SysUserRole;
import com.qmth.digit.print.business.mapper.SysRoleMapper;
import com.qmth.digit.print.business.service.SysRoleCourseService;
import com.qmth.digit.print.business.service.SysRolePrivilegeService;
import com.qmth.digit.print.business.service.SysRoleService;
import com.qmth.digit.print.business.service.SysUserRoleService;
import com.qmth.digit.print.common.enums.ExceptionResultEnum;
import com.qmth.digit.print.common.enums.RoleTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    @Autowired
    private SysUserRoleService sysUserRoleService;

    @Autowired
    private SysRolePrivilegeService sysRolePrivilegeService;

    @Autowired
    private SysRoleCourseService sysRoleCourseService;

    @Override
    public IPage<SysRole> list(RoleParam param) {
        Page<SysRole> page = new Page<>(param.getPageNumber(), param.getPageSize());
        IPage<SysRole> roleIPage = this.baseMapper.listPage(page, param);
        return roleIPage;
    }

    @Transactional
    @Override
    public boolean saveRole(SysRole role) {
        QueryWrapper<SysRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysRole::getSchoolId, role.getSchoolId()).eq(SysRole::getName, role.getName());
        SysRole sysRole = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(role.getId())) {
            if (sysRole != null) {
                throw ExceptionResultEnum.ERROR.exception("角色已存在");
            }
            role.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(role);
        }
        // 修改
        {
            if (sysRole != null && sysRole.getId() != role.getId()) {
                throw ExceptionResultEnum.ERROR.exception("角色已存在");
            }

            role.setUpdateTime(System.currentTimeMillis());

            //删除权限
            sysRolePrivilegeService.removeByRoleId(role.getId());

            //命题老师，删除科目
            if (role.getType().name().equals(RoleTypeEnum.QUESTION_TEACHER.name())) {
                sysRoleCourseService.removeByRoleId(role.getId());
            }
            isSuccess = this.updateById(role);

        }

        // 新增权限
        sysRolePrivilegeService.saveBatch(role);

        //新增科目
        if (role.getType().name().equals(RoleTypeEnum.QUESTION_TEACHER.name())) {
            sysRoleCourseService.saveBatch(role);
        }

        return isSuccess;
    }

    @Override
    public boolean enable(SysRole role) {
        UpdateWrapper<SysRole> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysRole::getEnable, role.getEnable()).eq(SysRole::getId, role.getId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean remove(Long id) {
        // 判断绑定用户
        List<SysUserRole> userRoleList = sysUserRoleService.listByRoleId(id);
        if (userRoleList != null && userRoleList.size() > 0) {
            throw ExceptionResultEnum.ERROR.exception("用户绑定，不能删除");
        }

        //删除权限
        sysRolePrivilegeService.removeByRoleId(id);
        return this.removeById(id);
    }

    @Override
    public List<Long> getUserRoles(Long userId) {
        QueryWrapper<SysUserRole> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUserRole::getUserId, userId);
        List<SysUserRole> sysUserRoles = sysUserRoleService.list(queryWrapper);
        List<Long> list = null;
        if (sysUserRoles != null && sysUserRoles.size() > 0) {
            list = sysUserRoles.stream().map(m -> m.getRoleId()).collect(Collectors.toList());
        }
        return list;
    }
}
