package com.qmth.digit.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.api.exception.ApiException;
import com.qmth.digit.print.business.bean.dto.UserDto;
import com.qmth.digit.print.business.bean.query.UserParam;
import com.qmth.digit.print.business.entity.BasicCourse;
import com.qmth.digit.print.business.entity.SysRole;
import com.qmth.digit.print.business.entity.SysUser;
import com.qmth.digit.print.business.entity.SysUserRole;
import com.qmth.digit.print.business.mapper.BasicCourseMapper;
import com.qmth.digit.print.business.mapper.SysRoleMapper;
import com.qmth.digit.print.business.mapper.SysUserMapper;
import com.qmth.digit.print.business.mapper.SysUserRoleMapper;
import com.qmth.digit.print.business.service.SysConfigService;
import com.qmth.digit.print.business.service.SysUserRoleService;
import com.qmth.digit.print.business.service.SysUserService;
import com.qmth.digit.print.business.util.security.Md5Utils;
import com.qmth.digit.print.common.enums.ExceptionResultEnum;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 用户表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements SysUserService {

    @Autowired
    private SysRoleMapper sysRoleMapper;

    @Autowired
    private BasicCourseMapper basicCourseMapper;

    @Autowired
    private SysUserRoleMapper sysUserRoleMapper;

    @Autowired
    private SysConfigService sysConfigService;

    @Autowired
    private SysUserRoleService sysUserRoleService;


    @Override
    public IPage<UserDto> list(UserParam param) {
        Page<UserDto> page = new Page<>(param.getPageNumber(), param.getPageSize());
        IPage<UserDto> userDtoIPage = this.baseMapper.listPage(page, param);
        if (userDtoIPage.getRecords().size() > 0) {
            userDtoIPage.getRecords().forEach(m -> {
                //角色
                List<SysRole> roles = sysRoleMapper.listRolesByUserId(m.getId());
                m.setRoles(roles);

                //课程
                List<BasicCourse> courses = basicCourseMapper.listCoursesByUserId(m.getId());
                m.setCourses(courses);
            });
        }
        return userDtoIPage;
    }

    @Transactional
    @Override
    public boolean saveUser(SysUser user) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUser::getLoginName, user.getLoginName());
        SysUser sysUser = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(user.getId())) {
            if (sysUser != null) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            String password = sysConfigService.getByKey("sys.user.initPassword");
            user.setPassword(Md5Utils.toMd5Hex(StringUtils.isNoneBlank(password) ? password : "123456"));
            user.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(user);
        }
        // 修改
        else {
            if (sysUser != null && user.getId() != sysUser.getId()) {
                throw ExceptionResultEnum.ERROR.exception("用户名已存在");
            }
            sysUser.setRealName(user.getRealName());
            sysUser.setMobileNumber(user.getMobileNumber());
            sysUser.setOrgId(user.getOrgId());
            sysUser.setUpdateTime(System.currentTimeMillis());
            isSuccess = this.updateById(sysUser);

            //删除角色
            UpdateWrapper<SysUserRole> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().eq(SysUserRole::getUserId, sysUser.getId());
            sysUserRoleMapper.delete(updateWrapper);
        }

        //新增用户-角色
        for (Long roleId : user.getRoleIds()) {
            SysUserRole sysUserRole = new SysUserRole();
            sysUserRole.setUserId(user.getId());
            sysUserRole.setRoleId(roleId);
            sysUserRoleMapper.insert(sysUserRole);
        }

        return isSuccess;
    }

    @Override
    public boolean enable(SysUser user) {
        UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(SysUser::getEnable, user.getEnable()).eq(SysUser::getId, user.getId());
        return this.update(updateWrapper);
    }

    @Override
    public boolean resetPassword(Long id) {
        String password = sysConfigService.getByKey("sys.user.initPassword");
        String md5Password = Md5Utils.toMd5Hex(StringUtils.isNoneBlank(password) ? password : "123456");

        SysUser user = this.getById(id);
        user.setPassword(md5Password);

        return this.updateById(user);
    }

    @Override
    public boolean updatePassword(Long id, String oldPassword, String password) {
        SysUser sysUser = this.getById(id);
        if(sysUser == null){
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }

        if(!StringUtils.equals(sysUser.getPassword(), oldPassword)){
            throw ExceptionResultEnum.ERROR.exception("原密码不正确");
        }
        String md5Password = Md5Utils.toMd5Hex(password);
        sysUser.setPassword(md5Password);
        return this.updateById(sysUser);
    }

    @Override
    public boolean bindRoles(Long id, Long... roleIds) {
        if(roleIds.length == 0){
            throw ExceptionResultEnum.ERROR.exception("请选择角色");
        }
        List<SysUserRole> list = new ArrayList<>();
        for (Long roleId : roleIds) {
            SysUserRole userRole = new SysUserRole();
            userRole.setUserId(id);
            userRole.setRoleId(roleId);
            list.add(userRole);
        }
        return sysUserRoleService.saveBatch(list);
    }
}
