package com.qmth.digit.print.business.util;

import com.qmth.digit.print.business.bean.ApiUser;
import com.qmth.digit.print.business.entity.SysUser;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.net.URLEncoder;

/**
 * request临时参数存取
 */
public class RequestUtils {

    private static final String REQUEST_API_USER_KEY = "api_user";

    private static final String REQUEST_OPERATION_LOG_KEY = "operation_log";


    public static ApiUser getApiUser(HttpServletRequest request) {
        Object obj = request.getAttribute(REQUEST_API_USER_KEY);
        if (obj != null && obj instanceof ApiUser) {
            return (ApiUser) obj;
        } else {
            SysUser sysUser = new SysUser();
            sysUser.setId(1L);
            sysUser.setSchoolId(101L);
            ApiUser apiUser = new ApiUser(sysUser);
            return apiUser;
//            return null;
        }
    }

    public static void setApiUser(HttpServletRequest request, ApiUser user) {
        request.setAttribute(REQUEST_API_USER_KEY, user);
    }

    /**
     * 设置 Cookie
     *
     * @param name   名称
     * @param value  值
     * @param maxAge 生存时间（单位秒）
     */
    public static void setCookie(HttpServletResponse response, String name, String value, int maxAge) {
        Cookie cookie = new Cookie(name, null);
        cookie.setPath("/");
        cookie.setMaxAge(maxAge);
        try {
            cookie.setValue(URLEncoder.encode(value, "utf-8"));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        response.addCookie(cookie);
    }

    /**
     * 获得指定Cookie的值
     *
     * @param name 名称
     * @return 值
     */
    public static String getCookie(HttpServletRequest request, String name) {
        return getCookie(request, null, name, false);
    }

    /**
     * 获得指定Cookie的值，并删除。
     *
     * @param name 名称
     * @return 值
     */
    public static String getCookie(HttpServletRequest request, HttpServletResponse response, String name) {
        return getCookie(request, response, name, true);
    }

    /**
     * 获得指定Cookie的值
     *
     * @param request  请求对象
     * @param response 响应对象
     * @param name     名字
     * @param isRemove 是否移除
     * @return 值
     */
    public static String getCookie(HttpServletRequest request, HttpServletResponse response, String name,
                                   boolean isRemove) {
        String value = null;
        Cookie[] cookies = request.getCookies();
        if (cookies != null) {
            for (Cookie cookie : cookies) {
                if (cookie.getName().equals(name)) {
                    try {
                        value = URLDecoder.decode(cookie.getValue(), "utf-8");
                    } catch (UnsupportedEncodingException e) {
                        e.printStackTrace();
                    }
                    if (isRemove) {
                        cookie.setMaxAge(0);
                        response.addCookie(cookie);
                    }
                }
            }
        }
        return value;
    }

    public static String getLog(HttpServletRequest request) {
        Object obj = request.getAttribute(REQUEST_OPERATION_LOG_KEY);
        if (obj != null && obj instanceof String) {
            return (String) obj;
        } else {
            return "";
        }
    }

    public static void setLog(HttpServletRequest request, String content) {
        request.setAttribute(REQUEST_OPERATION_LOG_KEY, content);
    }
}