package com.qmth.digit.print.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.digit.print.business.bean.ApiUser;
import com.qmth.digit.print.business.bean.dto.UserDto;
import com.qmth.digit.print.business.bean.query.BlurryParam;
import com.qmth.digit.print.business.bean.query.CourseParam;
import com.qmth.digit.print.business.bean.query.UserParam;
import com.qmth.digit.print.business.entity.BasicCourse;
import com.qmth.digit.print.business.service.BasicCourseService;
import com.qmth.digit.print.business.util.RequestUtils;
import com.qmth.digit.print.common.util.Result;
import com.qmth.digit.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;

import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 课程表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "课程Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.basic}/course")
@Aac(auth = BOOL.FALSE)
public class BasicCourseController {

    @Autowired
    private BasicCourseService basicCourseService;

    /**
     * 查询
     * @param param
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list(HttpServletRequest request, CourseParam param){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        IPage<BasicCourse> userDtos = basicCourseService.list(param);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 模糊查询
     * @param param
     * @return
     */
    @ApiOperation(value = "模糊查询")
    @RequestMapping(value = "/query", method = RequestMethod.GET)
    public Result query(HttpServletRequest request, BlurryParam param){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        param.setSchoolId(apiUser.getSchoolId());
        List<BasicCourse> userDtos = basicCourseService.list(param);
        return ResultUtil.ok(userDtos);
    }

    /**
     * 新增/修改
     * @param request
     * @param course
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody BasicCourse course){
        ApiUser apiUser = RequestUtils.getApiUser(request);
        course.setSchoolId(apiUser.getSchoolId());
        course.setCreateId(course.getId() == null ? apiUser.getSchoolId() : null);
        course.setUpdateId(course.getId() != null ? apiUser.getSchoolId() : null);
        boolean isSuccess = basicCourseService.saveCourse(course);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     * @param request
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(HttpServletRequest request, Long id){
        boolean isSuccess = basicCourseService.remove(id);
        return ResultUtil.ok(isSuccess);
    }
}

