package com.qmth.digit.print.api;


import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.digit.print.business.bean.ApiUser;
import com.qmth.digit.print.business.bean.dto.OrgDto;
import com.qmth.digit.print.business.entity.SysOrg;
import com.qmth.digit.print.business.service.SysOrgService;
import com.qmth.digit.print.business.util.RequestUtils;
import com.qmth.digit.print.common.util.Result;
import com.qmth.digit.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 学校组织架构 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "组织架构Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/org")
@Aac(auth = BOOL.FALSE)
public class SysOrgController {

    @Autowired
    private SysOrgService sysOrgService;

    /**
     * 查询机构树
     * @param request
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list(HttpServletRequest request) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        List<OrgDto> orgDtoList = sysOrgService.listOrgTree(apiUser.getSchoolId());
        return ResultUtil.ok(orgDtoList);
    }

    /**
     * 新增/修改
     * @param request
     * @param org
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody SysOrg org) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        org.setSchoolId(apiUser.getSchoolId());
        org.setCreateId(org.getId() == null ? apiUser.getUserData().getId() : null);
        org.setUpdateId(org.getId() != null ? apiUser.getUserData().getId() : null);
        boolean isSuccess = sysOrgService.saveOrg(org);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 启用/禁用
     * @param org
     * @return
     */
    @ApiOperation(value = "启用/禁用")
    @RequestMapping(value = "/enable", method = RequestMethod.POST)
    public Result enable(@RequestBody SysOrg org) {
        boolean isSuccess = sysOrgService.enable(org);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(Long id) {
        boolean isSuccess = sysOrgService.remove(id);
        return ResultUtil.ok(isSuccess);
    }

}

