package com.qmth.digit.print.api;


import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.digit.print.business.bean.ApiUser;
import com.qmth.digit.print.business.bean.dto.PrivilegeDto;
import com.qmth.digit.print.business.entity.SysPrivilege;
import com.qmth.digit.print.business.service.SysPrivilegeService;
import com.qmth.digit.print.business.util.RequestUtils;
import com.qmth.digit.print.common.util.Result;
import com.qmth.digit.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
 * 菜单权限表 前端控制器
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Api(tags = "菜单权限Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.sys}/privilege")
@Aac(auth = BOOL.FALSE)
public class SysPrivilegeController {

    @Autowired
    private SysPrivilegeService sysPrivilegeService;

    /**
     * 查询
     *
     * @param request
     * @return
     */
    @ApiOperation(value = "查询")
    @RequestMapping(value = "/list", method = RequestMethod.GET)
    public Result list(HttpServletRequest request) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        List<PrivilegeDto> privilegeDtoList = sysPrivilegeService.listOrgTree(apiUser.getSchoolId());
        return ResultUtil.ok(privilegeDtoList);
    }

    /**
     * 新增/修改
     *
     * @param request
     * @param privilege
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public Result save(HttpServletRequest request, @RequestBody SysPrivilege privilege) {
        ApiUser apiUser = RequestUtils.getApiUser(request);
        privilege.setSchoolId(apiUser.getSchoolId());
        privilege.setCreateId(privilege.getId() == null ? apiUser.getUserData().getId() : null);
        privilege.setUpdateId(privilege.getId() != null ? apiUser.getUserData().getId() : null);
        boolean isSuccess = sysPrivilegeService.savePrivilege(privilege);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 删除
     *
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @RequestMapping(value = "/remove", method = RequestMethod.POST)
    public Result remove(Long id) {
        boolean isSuccess = sysPrivilegeService.remove(id);
        return ResultUtil.ok(isSuccess);
    }

    /**
     * 角色已绑定权限列表
     *
     * @param roleId
     * @return
     */
    @ApiOperation(value = "角色已绑定权限列表")
    @RequestMapping(value = "/get_role_privileges", method = RequestMethod.GET)
    public Result getRolePrivileges(Long roleId) {
        List<Long> list = sysPrivilegeService.getRolePrivileges(roleId);
        return ResultUtil.ok(list);
    }
}

